/*  Измеритель КСВ на ардуинке.
*  Аппаратура: RobotDyn Nano v3, ATMega 328P
*  (c) RM1F 2019-2024
*  Версия с индикацией падающей волны для RL1L
*  и с переключателем антенн для него же
*/

// Для моей платы настройки:
// плата Arduino Nano
// процессор ATMega328 (old bootloader) - это важно!
// программатор ArduinoISP.org
// порт COM6

#include "Arduino.h" // for digitalRead and digitalWrite
#include <limits.h>

// ================== START CONFIGURATION SETTINGS ==================
// Скорость порта для отладки
#define BAUD_RATE           (115200)

// I/O pin definitions

// встроенный синий светодиод на плате: (13) for built-in LED on UNO
// #define LED_BUILTIN     (13)

// analog input pin for FORWARD power reading
#define FWD_PIN             (A0)
// analog input pin for REFLECTED power reading
#define REF_PIN             (A1)
// выходной контакт на прибор, будет использоваться ШИМ выход
#define OUT_PIN             (3)
// индикатор включения или перегрузки
#define LED_PIN             (7)
#define LED_FLASH_PERIOD    (250)
// контакт выбора режима: SWR или индикатор падающей волны
#define MODE_PIN            (5)

// входные контакты управления переключением антенн
#define ANT1_INPUT_PIN      (A2)
#define ANT2_INPUT_PIN      (A3)
#define ANT3_INPUT_PIN      (A4)
#define ANT4_INPUT_PIN      (A5)

// выходные контакты ключей переключения антенн
#define ANT1_OUTPUT_PIN     (8)
#define ANT2_OUTPUT_PIN     (9)
#define ANT3_OUTPUT_PIN     (10)
#define ANT4_OUTPUT_PIN     (11)

// максимальный SWR
#define MAX_SWR             (20)
// длительность цикла поллинга в миллисекундах
#define TIME_SLOT           (50)

// Maximum Arduino A/D reading; used only to scale the power readings.
const int AD_MAX = 1023;

// когда включать индикацию перегрузки ADC (90% max)
const int ADC_OVERLOAD = 920;

// не реагировать на слишком маленькие значения FWD и REF - могут быть наводки
const int ADC_LOW_LIMIT = 1;

// константы для сглаживания
const float alphaFwd = 0.5;
const float alphaRef = 0.5;

// измеренные значения падающей и отраженной волн
float forward = 0.;
float reflected = 0.;

// фиксируем момент перегрузки
bool overload = false;
int overloadCount = 0;

// текущая включенная антенна
uint8_t currentAnt = 0;

// forward declarations
int math_round(float x);
void setupAntSwitch();
void resetAntSwitch();
uint8_t antennaButton();
void selectAntenna(uint8_t ant);

//===================== собственно работа =======================

void setup()
{
   // put your setup code here, to run once:
    Serial.begin(BAUD_RATE);

    // the A/D inputs
    pinMode(FWD_PIN, INPUT);
    pinMode(REF_PIN, INPUT);
    // выход
    pinMode(OUT_PIN, OUTPUT);
    // индикатор включения
    pinMode(LED_PIN, OUTPUT);
    // пин выбора режима
    pinMode(MODE_PIN, INPUT_PULLUP);
   
    analogWrite(OUT_PIN, 0);
    digitalWrite(LED_PIN, HIGH);
    digitalWrite(MODE_PIN, HIGH);

    setupAntSwitch();
}

void loop()
{
    // измеренные значения падающей и отраженной волн со сглаживанием
    forward = (alphaFwd * analogRead(FWD_PIN)) + ((1.0 - alphaFwd) * forward);
    reflected = (alphaRef * analogRead(REF_PIN)) + ((1.0 - alphaRef) * reflected);

    float ratio = 0.;
   
    if (digitalRead(MODE_PIN) == LOW)
    {   // режим измерения КСВ
        // что вывести на приборчик - соотношение ref/fwd
        if (reflected <= ADC_LOW_LIMIT)
        {
            ratio = 0.;
        }
        else if (reflected >= forward)
        {
            ratio = 1.0;
        }
        else
        {
            ratio =  reflected / forward;
        }
    }
    else
    {   // режим измерения падающей волны
        ratio = forward / static_cast<float>(AD_MAX);
    }
    // индикация перегрузки
    overload = (forward >= ADC_OVERLOAD || reflected >= ADC_OVERLOAD);
    if (overload)
    {
        overloadCount = ++overloadCount % 10;
        if (overloadCount == 0)
            digitalWrite(LED_PIN, digitalRead(LED_PIN)^1);
    }
    else
    {
        digitalWrite(LED_PIN, HIGH);
        overloadCount = 0;
       
        // ШИМ выходного пина
        int val = math_round(ratio * UCHAR_MAX);
        analogWrite(OUT_PIN, val);
    }

    // обработка кнопок управления антеннами
    uint8_t ant = antennaButton();
    if (ant && ant != currentAnt)
    {
        selectAntenna(ant);
    }
    
    delay(TIME_SLOT);
}

int math_round(float x)
{
    return (int)(x + 0.5);
}

void setupAntSwitch()
{
    pinMode(ANT1_INPUT_PIN, INPUT_PULLUP);
    pinMode(ANT2_INPUT_PIN, INPUT_PULLUP);
    pinMode(ANT3_INPUT_PIN, INPUT_PULLUP);
    pinMode(ANT4_INPUT_PIN, INPUT_PULLUP);

    digitalWrite(ANT1_INPUT_PIN, HIGH);
    digitalWrite(ANT2_INPUT_PIN, HIGH);
    digitalWrite(ANT3_INPUT_PIN, HIGH);
    digitalWrite(ANT4_INPUT_PIN, HIGH);

    pinMode(ANT1_OUTPUT_PIN, OUTPUT);
    pinMode(ANT2_OUTPUT_PIN, OUTPUT);
    pinMode(ANT3_OUTPUT_PIN, OUTPUT);
    pinMode(ANT4_OUTPUT_PIN, OUTPUT);
    
    resetAntSwitch();
    currentAnt = 0;
}

void resetAntSwitch()
{
    digitalWrite(ANT1_OUTPUT_PIN, LOW);  
    digitalWrite(ANT2_OUTPUT_PIN, LOW);  
    digitalWrite(ANT3_OUTPUT_PIN, LOW);  
    digitalWrite(ANT4_OUTPUT_PIN, LOW);  
}

// возврат: 1..4 - номер нажатой кнопки управления антенной
// 0, если не нажата ни одна кнопка, или нажаты несколько кнопок сразу (недопустимая комбинация)
uint8_t antennaButton()
{
    bool a1 = (digitalRead(ANT1_INPUT_PIN) == LOW);
    bool a2 = (digitalRead(ANT2_INPUT_PIN) == LOW);
    bool a3 = (digitalRead(ANT3_INPUT_PIN) == LOW);
    bool a4 = (digitalRead(ANT4_INPUT_PIN) == LOW);

    if (a1 && !a2 && !a3 && !a4)
        return 1;
    if (a2 && !a1 && !a3 && !a4)
        return 2;
    if (a3 && !a1 && !a2 && !a4)
        return 3;
    if (a4 && !a1 && !a2 && !a3)
        return 4;
        
    return 0;
}

void selectAntenna(uint8_t ant)
{
      resetAntSwitch();
      switch(ant)
      {
          case 1:  digitalWrite(ANT1_OUTPUT_PIN, HIGH); break;
          case 2:  digitalWrite(ANT2_OUTPUT_PIN, HIGH); break;
          case 3:  digitalWrite(ANT3_OUTPUT_PIN, HIGH); break;
          case 4:  digitalWrite(ANT4_OUTPUT_PIN, HIGH); break;
          default:
              /* none */;
      }
}
